#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import sys
import syslog
from enum import Enum

from sysrepo_interfaces import sysrepo_interfaces3 as api


BIND_INTERFACE_FILE = '/kubernetes/bind-interface-file'
K8S_ENABLED_FILE = '/config/k8s-enabled'
CENTRAL_BOX_FILE = '/config/central-box'

class ValidateErrors(Enum):
    OK = 0
    NOT_FOUND = -1
    IP_DUP_FOUND = -2
    METRIC_DUP_FOUND = -3
    NO_MGMNT = -4
    NOVALUE_IN_SYSREPO = -5
    TOO_MANY_VLANS = -6
    NO_METRIC = -7
    BAD_METRIC = -8


def getErrorAsText(error, optional_text='', optional_text1=''):
    errorText = None

    if error == ValidateErrors.OK:
        errorText = ''
    elif error == ValidateErrors.NOT_FOUND:
        errorText = "Validation Error: Interface '%s' does not exist!" % optional_text
    elif error == ValidateErrors.IP_DUP_FOUND:
        errorText = "Validation Error: Another interface - '%s' with the same IP found!" % optional_text
    elif error == ValidateErrors.METRIC_DUP_FOUND:
        errorText = "Validation Error: Another interface - '%s' has the same metric!\nThis will result in default routes with the same metric!" % optional_text
    elif error == ValidateErrors.NO_MGMNT:
        errorText = 'Validation Error: At least one management should be present!'
    elif error == ValidateErrors.NOVALUE_IN_SYSREPO:
        errorText = 'Validation Error: Could not retrieve interface data!'
    elif error == ValidateErrors.TOO_MANY_VLANS:
        errorText = "Validation Error: Interface '%s' can not exceed 5 vlans!" % optional_text
    elif error == ValidateErrors.NO_METRIC:
        errorText = 'Validation Error: Please set the metric first!'
    elif error == ValidateErrors.BAD_METRIC:
        errorText = "Validation Error: Metric should be in range 0 - 200!"

    return errorText


# Get value from sysrepo, but the idea here is to decrease complexity of rule's logic all None values to be
# represent as empty strings
def getSysrepoValue(iface_data, valueName=''):
    if valueName is not None and valueName != '':
        value = iface_data.get(valueName)
        if value is not None:
            return value
    return ''


def validate_eth_node(instance, name, **additional_parameters):
    ifaces_data = instance.get_interfaces()

    ret_val = ''  # Emulated previous rule -> Ok

    # Merged values from function parameters and sysrepo
    current_ipv4 = additional_parameters.get('ipv4', '')
    current_ipv6 = additional_parameters.get('ipv6', '')
    current_bootproto = additional_parameters.get('bootproto', '')
    current_gateway = additional_parameters.get('gateway', '')
    current_management = additional_parameters.get('management', '')
    current_metric = additional_parameters.get('metric', '')

    ifname, vlan = name.split('.')
    # ============ Rule - Presence ============
    if ret_val == '':  # Check result from previous rule
        ret_val = getErrorAsText(ValidateErrors.NOT_FOUND, ifname)  # prepare for not found error
        for iface_data in ifaces_data.values(): # validate that base ethernet address exists
            try:
                if ifname == getSysrepoValue(iface_data, 'name') and 'ethernet' == getSysrepoValue(iface_data, 'type'):
                    ret_val = getErrorAsText(ValidateErrors.OK)
                    break
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

    # ============ Rule - Number of allowed VLANs ============
    if ret_val == '':  # Check result from previous rule
        count = 0
        for iface_data in ifaces_data.values(): # count existing vlans on interface are less than 5
            try:
                if ifname in getSysrepoValue(iface_data, 'name') and 'vlan' == getSysrepoValue(iface_data, 'type'):
                    if vlan == getSysrepoValue(iface_data, 'name').split('.')[-1]:
                        count+=1
                        # Use the rule for loading of missing data values for that interface name:
                        if current_bootproto == '':
                            current_bootproto = getSysrepoValue(iface_data, 'bootproto')
                        if current_metric == '':
                            current_metric = getSysrepoValue(iface_data, 'metric')

                        # IPv4 address and the gateway are loaded (if not empty) for comparison/validation
                        if current_gateway == '':
                            current_gateway = getSysrepoValue(iface_data, 'gateway')
                        if current_ipv4 == '':
                            current_ipv4 = getSysrepoValue(iface_data, 'ipv4-prefix')

                        break
                    else:
                        count+=1
            except:
                ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                break

        if count > 5:
            ret_val = getErrorAsText(ValidateErrors.TOO_MANY_VLANS, ifname)

    # ============ Rule - Presense of metric ============
    if ret_val == '':  # Check result from previous rule
        if current_metric == '' and (('none' in current_bootproto and current_gateway != '') or current_bootproto == 'dhcp'):
            ret_val = getErrorAsText(ValidateErrors.NO_METRIC)

    # ============ Rule - Metric out of range 0 - 200 ============
    if ret_val == "":  # Check result from previous rule
        if (current_metric == '' or not (0 <= int(current_metric) <= 200)):
            ret_val = getErrorAsText(ValidateErrors.BAD_METRIC)  # prepare for metric out of range

    # Compare subnet addresses in binary
    if current_gateway != '' and current_ipv4 != '' and (current_bootproto in ['none', 'dhcp6']):
        netmask = int(current_ipv4.split('/')[1])
        gateway_list = current_gateway.split('.')

        for i in range(len(gateway_list)):
            curr_oct = gateway_list[i]
            gateway_list[i] = '{0:08b}'.format(int(curr_oct))

        bin_gateway = ''.join(gateway_list)
        bin_gateway = bin_gateway[0: netmask]
        for i in range(netmask, 31):
            bin_gateway = bin_gateway + '0'

        ip = current_ipv4.split('/')[0]
        ip_list = ip.split('.')

        for i in range(len(ip_list)):
            curr_oct = ip_list[i]
            ip_list[i] = '{0:08b}'.format(int(curr_oct))

        bin_ip = ''.join(ip_list)
        bin_ip = bin_ip[0: netmask]
        for i in range(netmask, 31):
            bin_ip = bin_ip + '0'

        if bin_ip != bin_gateway:
            if additional_parameters.get('ipv4', '') != '':
                print(f'Make sure to change the gateway.\nThe current gateway ({current_gateway}) is not in the same subnet as the IPv4 address.')
            elif additional_parameters.get('gateway', '') != '':
                print(f'Make sure to change the IPv4 address.\nThe current IPv4 address ({current_ipv4}) is not in the same subnet as the gateway.')

    # ============ Rule - Management presence ============
    if ret_val == '':  # Check result from previous rule
        # If there is disabling of management -> check how many management interfaces are present
        # At least one interface needs to be present management
        if current_management == '0':
            mgmnt_count = 0
            for iface_data in ifaces_data.values():
                if True == getSysrepoValue(iface_data, 'management'):
                    if_type = getSysrepoValue(iface_data, 'type')
                    if if_type in ['ethernet', 'lte', 'vlan']:
                        mgmnt_count = mgmnt_count + 1

            # If there is one, check the request for disabling for which one is
            if mgmnt_count == 1:
                for iface_data in ifaces_data.values():
                    try:
                        if name == getSysrepoValue(iface_data, 'name') \
                           and getSysrepoValue(iface_data, 'management') == True:
                            ret_val = getErrorAsText(ValidateErrors.NO_MGMNT)  # prepare for not present management error
                            break
                    except:
                        ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                        break

    # ============ Rule - IP Duplication ============
    if ret_val == '':  # Check result from previous rule
        # If the user is trying to set IPv4 address, but the interface is in DHCP or DHCP46 mode, and error will occur
        # current_ipv4/current_ipv6 are not used here, because if the user does not enter a new IPv4/v6 address, there is no need for IP duplication verification
        if additional_parameters.get('ipv4', '') != '':
            if current_bootproto in ['dhcp6', 'none']:
                for iface_data in ifaces_data.values():
                    try:
                        if name != getSysrepoValue(iface_data, 'name') and (current_ipv4 == getSysrepoValue(iface_data, 'ipv4-prefix')):
                            ret_val = getErrorAsText(ValidateErrors.IP_DUP_FOUND, getSysrepoValue(iface_data, 'name'))
                            break
                    except:
                        ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                        break
            else:
                ret_val = f'Cannot change IP! The interface bootproto is "{current_bootproto}".\nYou can change it via the bootproto command.'

        # If the user is trying to set IPv6 address, but the interface is in DHCP6 or DHCP46 mode, and error will occur
        if additional_parameters.get('ipv6', '') != '':
            if current_bootproto in ['dhcp', 'none']:
                for iface_data in ifaces_data.values():
                    try:
                        if name != getSysrepoValue(iface_data, 'name') and current_ipv6 == getSysrepoValue(iface_data, 'ipv6-prefix'):
                            ret_val = getErrorAsText(ValidateErrors.IP_DUP_FOUND, getSysrepoValue(iface_data, 'name'))
                            break
                    except:
                        ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                        break
            else:
                ret_val = f'Cannot change IP! The interface bootproto is "{current_bootproto}".\nYou can change it via the bootproto command.'

    # ============ Rule - Metric Duplication ============
    if ret_val == '':  # Check result from previous rule
        # Search for duplicated metrics in follow cases:
        # 1. if bootproto == dhcp -> check metric of other interfaces (in both cases of their bootproto == dhcp and bootproto == none and present gateway)
        # 2. if bootproto == none and present gateway -> check metric of other interfaces (in both cases of their bootproto == dhcp and bootproto == none and present gateway)
        if (current_bootproto == 'none' and current_gateway != '') or (current_bootproto != 'none'):
            for iface_data in ifaces_data.values():
                try:
                    # Check other interfaces for conflicts
                    if name == getSysrepoValue(iface_data, 'name') or \
                           current_metric == '' or \
                           getSysrepoValue(iface_data, 'metric') == '' or \
                           int(current_metric) != int(getSysrepoValue(iface_data, 'metric')):
                        continue
                    if getSysrepoValue(iface_data, 'bootproto') == 'dhcp' or (getSysrepoValue(iface_data, 'bootproto') == 'none' and getSysrepoValue(iface_data, 'gateway') != ''):
                            ret_val = getErrorAsText(ValidateErrors.METRIC_DUP_FOUND, getSysrepoValue(iface_data, 'name'))

                            # Printing all interfaces with their metric values, so  the user can set the correct metric for the current interface
                            duplicate_name = getSysrepoValue(iface_data, 'name')
                            print('Used metrics / gateways:')
                            for iface in ifaces_data.values():
                                try:
                                    selected = '      '
                                    if (getSysrepoValue(iface, 'metric') != '' \
                                        and getSysrepoValue(iface, 'name') != name \
                                        and (getSysrepoValue(iface, 'bootproto') == 'none' and getSysrepoValue(iface, 'gateway') != '' \
                                             or getSysrepoValue(iface, 'bootproto') != 'none')):
                                        if (getSysrepoValue(iface, 'name') == duplicate_name):
                                            selected = 'ERR-> '
                                        if (getSysrepoValue(iface, 'gateway') == ''):
                                            print(selected, "Interface '", getSysrepoValue(iface, 'name'), "' metric ", getSysrepoValue(iface, 'metric'), sep='')
                                        else:
                                            print(selected, "Interface '", getSysrepoValue(iface, 'name'), "' metric ", getSysrepoValue(iface, 'metric'), ' gateway ', getSysrepoValue(iface, 'gateway'), sep='')
                                except:
                                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                                    break
                            break
                except:
                    ret_val = getErrorAsText(ValidateErrors.NOVALUE_IN_SYSREPO)
                    break

    return ret_val


def check_bind_interface():
    try:
        with open(BIND_INTERFACE_FILE, 'r') as file:
            content = file.read()
            return content
    except FileNotFoundError:
        return 'File not found.'
    except Exception as e:
        return f'An error occurred: {e}'


def set_eth_node(name, param, value):
    if not param or not value:
        print('Error applying ethernet configuration.')
        return

    interfaces = api.SysrepoInterfaces(app_name='set vlans')
    create_vlan_iface(interfaces, name)

    error_message = ''

    if param == 'bootproto':
        error_message = validate_eth_node(interfaces, name, bootproto=value)
    elif param == 'gateway':
        error_message = validate_eth_node(interfaces, name, gateway=value)
    elif param == 'ipv4-prefix':
        error_message = validate_eth_node(interfaces, name, ipv4=value)
    elif param == 'ipv6-prefix':
        error_message = validate_eth_node(interfaces, name, ipv6=value)
    elif param == 'management':
        error_message = validate_eth_node(interfaces, name, management=value)
    elif param == 'metric':
        error_message = validate_eth_node(interfaces, name, metric=value)
    else:
        error_message = validate_eth_node(interfaces, name)

    if error_message == '':
        if param == 'enable':
            if int(value) == 1:
                interfaces.set_interface_param(name, 'enable', True)
            else:
                interfaces.set_interface_param(name, 'enable', False)
        elif param == 'bootproto':
            # Deleting static values for ip address, dns, gateway
            if value == 'dhcp':
                interfaces.unset_interface_param(name, 'ipv4-prefix')
                interfaces.unset_interface_param(name, 'gateway')
                interfaces.unset_interface_param(name, 'dns')
            elif value == 'dhcp6':
                interfaces.unset_interface_param(name, 'ipv6-prefix')
                interfaces.unset_interface_param(name, 'gateway6')
                interfaces.unset_interface_param(name, 'dns6')
            elif value == 'dhcp46':
                interfaces.unset_interface_param(name, 'ipv4-prefix')
                interfaces.unset_interface_param(name, 'gateway')
                interfaces.unset_interface_param(name, 'dns')
                interfaces.unset_interface_param(name, 'ipv6-prefix')
                interfaces.unset_interface_param(name, 'gateway6')
                interfaces.unset_interface_param(name, 'dns6')

            interfaces.set_interface_param(name, param, value)
        elif param == 'ipv4-prefix':
            if value != '0.0.0.0/0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == 'ipv6-prefix':
            if value != '0000::/0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == 'gateway':
            if value != '0.0.0.0':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == 'gateway6':
            if value != '::':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)
        elif param == 'ip-passthrough':
            if not is_k8s_enabled():
                interfaces.set_interface_param(name, param, True if int(value) == 1 else False)
            else:
                bind_interface = check_bind_interface()
                if bind_interface.startswith('br-data') or bind_interface.startswith('br-tun'):
                    bind_interface = bind_interface.replace('br-', '').strip()
                    if bind_interface == name:
                        print('ip-passthrough cannot be set on Kubernetes bind-interface!')
                        return
                    interfaces.set_interface_param(name, param, True if int(value) == 1 else False)
                        
        elif (param == 'management' or param == 'link-state-propagation'):
            interfaces.set_interface_param(name, param, True if int(value) == 1 else False)
        elif param == 'trace-host':
            if value != 'none':
                interfaces.set_interface_param(name, param, value)
            else:
                interfaces.unset_interface_param(name, param)

        # Unsetting a parameter
        # value -> the parameter that needs to be unset
        elif param == 'no':
            if value == 'ipv4':
                interfaces.unset_interface_param(name, 'ipv4-prefix')
            elif value == 'ipv6':
                interfaces.unset_interface_param(name, 'ipv6-prefix')
            else:
                interfaces.unset_interface_param(name, value)

        # Setting all other parameters that do not require additional validation/configuration
        else:
            interfaces.set_interface_param(name, param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def set_static_ipv4_eth_node(name, newip, newgw):
    if not newip or not newgw:
        print('Error applying static ipv4 ethernet configuration.')
        return

    interfaces = api.SysrepoInterfaces(app_name='vlan set_static_ipv4')
    create_vlan_iface(interfaces, name)

    error_message = validate_eth_node(interfaces, name, bootproto='none', gateway=newgw, ipv4=newip)
    if error_message != '':
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)
        return
        
    error_message = validate_eth_node(interfaces, name)
    if error_message != '':
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)
        return

    if error_message == '':
        interfaces.set_interface(name)
        interfaces.set_interface_param(name, 'type', 'vlan')
        interfaces.set_interface_param(name, 'bootproto', 'none')
        interfaces.set_interface_param(name, 'ipv4-prefix', newip)
        interfaces.set_interface_param(name, 'gateway', newgw)
        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def set_trace_host_opts(name, param, value, probe=None):
    if not param or not value:
        print('Error applying trace host configuration.')
        return

    interfaces = api.SysrepoInterfaces(app_name='set vlan trace host opts')
    create_vlan_iface(interfaces, name)

    error_message = ''
    # Validations should be added for interval, count, timeout, ..

    if error_message == '':
        interfaces.set_interface(name)

        # Setting boolean parameters
        if param == 'trace-state-propagation':
            interfaces.set_container_param(name, 'tracehost', param, True if int(value) == 1 else False)
        elif param == 'enable':
            interfaces.set_container_param(name, 'tracehost', param, True if int(value) == 1 else False)

        # Unsetting a parameter
        elif param == 'no':
            if value != '':
                interfaces.unset_container_param(name, 'tracehost', value)

        # Setting all other parameters that do not require additional validation/configuration
        else:
            interfaces.set_container_param(name, 'tracehost', param, value)

        interfaces.commit()
    else:
        syslog.syslog(syslog.LOG_ERR, error_message)
        print(error_message)


def is_k8s_enabled():
    return os.path.isfile(K8S_ENABLED_FILE) or os.path.isfile(CENTRAL_BOX_FILE)


def create_vlan_iface(sri, ifname):
    if not check_interface(sri, ifname):
        sri.set_interface(ifname)
        sri.set_interface_param(ifname, 'type', 'vlan')


def check_interface(sri, ifname):
    if ifname in sri.get_interfaces().keys():
        return True
    else:
        return False


def delete_interface(ifname):
    interfaces = api.SysrepoInterfaces(app_name='delete vlan')
    if check_interface(interfaces, ifname):
        interfaces.unset_interface(ifname)
        interfaces.commit()


def set_smart_pp_param(name, protocol , param, value):
    interfaces = api.SysrepoInterfaces(app_name='set smart protocol passthrough')

    if param == 'enable' and value != '""':
        value = int(value) == 1

    interfaces.set_smart_pp_param(name, protocol, param, value)
    interfaces.commit()


def del_smart_pp(name, protocol):
    interfaces = api.SysrepoInterfaces(app_name='del smart protocol passthrough')
    interfaces.del_smart_pp(name, protocol)
    interfaces.commit()


if __name__ == '__main__':
    argc = len(sys.argv)

    if 3 > argc:
        syslog.syslog(syslog.LOG_ERR, 'No enough arguments. Exit!')
        sys.exit(1)
    elif sys.argv[1] == 'set_static_ipv4_eth_node':
        set_static_ipv4_eth_node(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == 'set_eth_node_int':
        set_eth_node(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == 'set_trace_host_opts':
        set_trace_host_opts(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == 'delete_int':
        delete_interface(sys.argv[2])
    elif sys.argv[1] == 'set_smart_pp_param':
        if sys.argv[3] == 'delete':
            del_smart_pp(sys.argv[2], sys.argv[4])
        else:
            set_smart_pp_param(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5])
    else:
        syslog.syslog(syslog.LOG_ERR, 'Unknown argument \"%s\"' % sys.argv[1])
        sys.exit(1)

    sys.exit(0)
