#!/usr/bin/env python3

import sys
import syslog
import re
import os
import subprocess
import shlex
import signal
import xml.etree.ElementTree as ET
import vppapi_if.vppapi_if as vppif
from time import sleep


VPP_SH_TAP_COMMAND = "sudo /usr/bin/vppctl show tap | awk '/^Interface:/ {interface=$2} /name \"eth/ {print interface}'"
VPP_SH_VTI_COMMAND = "sudo /usr/bin/vppctl show interface | grep VirtualEthernet | awk '{print $1}'"
VPP_SH_HARD_COMMAND = 'sudo /usr/bin/vppctl show hard '
CLASS_NET_COMMAND = 'sudo cat /sys/class/net/'

LABELS = ["name", "mac", "ip_address", "state", "speed", "duplex"]
PACKET_LABELS = ["rx packets", "rx bytes", "tx packets", "tx bytes"]
PACKET_NAT_LABELS = ["rx packets", "rx bytes"]
vpp_if = vppif.VppApiInterface('show-commands')


PACKET_LABELS_STATISTICS = {
    "packets": "Packets",
    "bytes": "Bytes",
    "error": "Errors",
    "drops": "Drops",
    "frame drops": "Frame Drops",
    "oversize": "Oversize",
    "crs errors": "CRC errors",
    "collisions": "Collisions",
}


MAX_LABEL_WIDTH = 20
MAX_VALUE_WIDTH = 20


SPEED_LABELS = {
    10: '10Mbps',
    100: '100Mbps',
    1000: '1Gbps',
    10000: '10Gbps',
    25000: '25Gbps',
    40000: '40Gbps',
    100000: '100Gbps',
    200000: '200Gbps',
    400000: '400Gbps'
}

def signal_handler(_, __):
    print('\nTerminated by the user!')
    sys.exit(1)  # Exit the program with a status code of 1


def format_speed(speed):
    for speed_value, label in SPEED_LABELS.items():
        if int(speed) == speed_value:
            return label
    return 'unknown'


def get_br_data_ip(data):
    if data.startswith('tun'):
        command_ip = f"/sbin/ifconfig {data} | awk '/inet / {{print $2}}'"
    else:
        command_ip = f"/sbin/ifconfig br-{data} | awk '/inet / {{print $2}}'"
    process = subprocess.Popen(command_ip, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output, error = process.communicate()
    if error:
        error_message = "Error: " + error.decode("utf-8")
        syslog.syslog(syslog.LOG_ERR, error_message)
        return None
    ip_address = output.decode().strip()
    return ip_address


def get_all_intf_names():
    try:
        output = subprocess.check_output(['sysrepocfg', '--datastore=running', '--format=xml',
                                          "--xpath=/ucpe:config/interfaces:interface/name",
                                          '--export']).decode().strip()
        root = ET.fromstring(output)
        namespaces = {'ucpe': 'urn:ucpe', 'interfaces': 'urn:interfaces'}
        intf_names = [elem.text.strip() for elem in root.findall(".//{urn:interfaces}name", namespaces)]
        return intf_names

    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, f"Failed to obtain interface names. Error: {e}.")
        return []


def get_intf_mode(intf):
    intf = str(intf).split('.')[0]
    try:
        output = subprocess.check_output(['sysrepocfg', '--datastore=running', '--format=xml',
                                          f"--xpath=/ucpe:config/interfaces:interface[name='{intf}']/mode",
                                          '--export']).decode().strip()
        root = ET.fromstring(output)
        namespaces = {'ucpe': 'urn:ucpe', 'interfaces': 'urn:interfaces'}
        intf_mode = root.find(".//{urn:interfaces}mode", namespaces).text.strip()
        return intf_mode

    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, f"The interface mode can not be obtained. Error: {e}.")
        return None


def get_intf_state(output, intf):
    if intf.startswith('tap'):
        state_matches = re.findall(r'\b(up|down)\b', output)
        if state_matches:
            state = "up" if "up" in state_matches else "down"
        else:
            state = "down"
    else:
        #try to get link state, if fails then fallback to ifconfig.
        state = get_intf_link_state(intf)
        if state != "unknown":
            return state

        ifconfig_process = subprocess.run(['/sbin/ifconfig', intf], capture_output=True, text=True)
        if ifconfig_process.returncode == 0:
            if "UP" in ifconfig_process.stdout:
                state = "up"
            else:
                state = "down"
        else:
            state = "unknown"

    return state


def get_intf_link_state(intf):
    # Check if carrier file exists
    carrier_path = f"/sys/class/net/{intf}/carrier"
    if os.path.exists(carrier_path):
        carrier_output = subprocess.run(['cat', carrier_path], capture_output=True, text=True)
        if carrier_output.returncode == 0:
            carrier_state = carrier_output.stdout.strip()
            if carrier_state == "1":
                state = 'up'
            else:
                state = 'down'
        else:
            state = 'unknown'
    else:
        state = 'unknown'

    return state


def get_mac_address(intf):
    if intf.startswith('tap') or intf.startswith('VirtualEthernet'):
        mac_command = (f'sudo /usr/bin/vppctl show hardware-interface {intf} '
                       f'| grep "Ethernet addr" |  tr -s " " | cut -f4 -d " " | cut -c 1-17')
        try:
            output = subprocess.check_output(mac_command, shell=True, stderr=subprocess.PIPE)
            mac_address = output.decode('utf-8').strip()
        except subprocess.CalledProcessError as e:
            if "No such file or directory" in e.stderr.decode('utf-8'):
                syslog.syslog(syslog.LOG_ERR, f'File not found for MAC address of {intf}')
                mac_address = 'unknown'
            else:
                syslog.syslog(syslog.LOG_ERR, f'Can not obtain the MAC address for {intf} due to {e}')
                mac_address = 'unknown'
    elif intf.startswith('vti') or intf.startswith('tun'):
        mac_address = 'unknown'
    elif intf.startswith('data') and '.' in intf:
        mac_output = subprocess.check_output(shlex.split(CLASS_NET_COMMAND + f"br-{intf}" + '/address'),
                                             stderr=subprocess.PIPE).decode('utf-8')
        mac_address = mac_output.strip()
    else:
        try:
            mac_output = subprocess.check_output(shlex.split(CLASS_NET_COMMAND + intf + '/address'),
                                                 stderr=subprocess.PIPE).decode('utf-8')
            mac_address = mac_output.strip()
            if mac_address == '':
                mac_address = 'unknown'
        except subprocess.CalledProcessError as e:
            if "No such file or directory" in e.stderr.decode('utf-8'):
                syslog.syslog(syslog.LOG_ERR, f'File not found for MAC address of {intf}')
                mac_address = 'unknown'
            else:
                syslog.syslog(syslog.LOG_ERR, f'Can not obtain the MAC address for {intf} due to {e}')
                mac_address = 'unknown'
        except Exception as e:
            syslog.syslog(syslog.LOG_ERR, f'Error occurred while getting MAC address for {intf}: {e}')
            mac_address = 'unknown'
    return mac_address


def get_intf_speed(intf):
    if '.' in intf:
        speed = 'unknown'
        return speed
    try:
        speed_output = subprocess.check_output(shlex.split(CLASS_NET_COMMAND + intf + '/speed'),
                                               stderr=subprocess.PIPE).decode('utf-8')
        speed = format_speed(speed_output.strip())
    except subprocess.CalledProcessError as e:
        if "No such file or directory" in e.stderr.decode('utf-8'):
            syslog.syslog(syslog.LOG_ERR, f'File not found for speed of {intf}')
            return 'unknown'
        else:
            syslog.syslog(syslog.LOG_ERR, f'Can not obtain the speed for {intf} due to {e}')
            return 'unknown'
    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, f'Error occurred while getting speed for {intf}: {e}')
        return 'unknown'
    return speed


def get_intf_duplex(intf):
    if '.' in intf:
        duplex = 'unknown'
        return duplex
    try:
        duplex_output = subprocess.check_output(shlex.split(CLASS_NET_COMMAND + intf + '/duplex'),
                                                stderr=subprocess.PIPE).decode('utf-8')
        duplex = duplex_output.strip()
    except subprocess.CalledProcessError as e:
        if "No such file or directory" in e.stderr.decode('utf-8'):
            syslog.syslog(syslog.LOG_ERR, f'File not found for duplex of {intf}')
            return 'unknown'
        else:
            syslog.syslog(syslog.LOG_ERR, f'Can not obtain the duplex for {intf} due to {e}')
            return 'unknown'
    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, f'Error occurred while getting duplex for {intf}: {e}')
        return 'unknown'
    return duplex


def get_intf_stats_vpn_vlan(intf):
    stats = {}
    stats_path = f"/sys/class/net/{intf}/statistics/"
    try:
        with open(os.path.join(stats_path, 'rx_packets'), 'r') as file:
            stats['rx_packets'] = file.read().strip()
        with open(os.path.join(stats_path, 'rx_bytes'), 'r') as file:
            stats['rx_bytes'] = file.read().strip()
        with open(os.path.join(stats_path, 'tx_packets'), 'r') as file:
            stats['tx_packets'] = file.read().strip()
        with open(os.path.join(stats_path, 'tx_bytes'), 'r') as file:
            stats['tx_bytes'] = file.read().strip()
    except FileNotFoundError:
        syslog.syslog(syslog.LOG_ERR, f'Statistics files not found for interface {intf}')
    except Exception as e:
        syslog.syslog(syslog.LOG_ERR, f'Error occurred while getting stats for {intf}: {e}')
    return stats.get('rx_packets', '0'), stats.get('rx_bytes', '0'), stats.get('tx_packets', '0'), stats.get('tx_bytes', '0')


def get_initial_interface_data(intfs):
    intf_statistics = {}

    for intf in intfs:
        name = intf
        statistics_output = get_statistics_output(intf)

        ip_address = get_br_data_ip(intf)

        # When the physical interfaces are in dpdk mode - obtain their state, speed and duplex via vppctl
        intf_mode = get_intf_mode(intf)
        if intf.startswith('data') and intf_mode == 'dpdk':

            hard_output = subprocess.check_output(shlex.split(VPP_SH_HARD_COMMAND + intf)).decode('utf-8')
            state_pattern = r'carrier (\w+)'
            speed_pattern = r'Link speed: (\d+ Mbps)'
            duplex_pattern = r'(\w+)\s+duplex'
            mac_pattern = r'Ethernet address\s+(\w{2}(?::\w{2}){5})'
            state = extract_value(state_pattern, hard_output)
            speed = extract_value(speed_pattern, hard_output)
            if not speed:
                speed_pattern = r'Link speed: (\d+ Gbps)'
                speed = extract_value(speed_pattern, hard_output)
                if speed:
                    speed = str(speed).replace(' ', '')
            if not speed:
                speed = "unknown"
            duplex = extract_value(duplex_pattern, hard_output)
            if not duplex:
                duplex = "unknown"
            mac = extract_value(mac_pattern, hard_output)
        else:
            state = get_intf_state(statistics_output, intf)
            speed = get_intf_speed(intf)
            duplex = get_intf_duplex(intf)
            mac = get_mac_address(intf)

        if intf.startswith(('vti', 'tun', 'vlan')):
            rx_packets, rx_bytes, tx_packets, tx_bytes = get_intf_stats_vpn_vlan(intf)
        else:
            rx_packets, rx_bytes, tx_packets, tx_bytes = extract_statistics(statistics_output)

        # Store primary statistics in dictionary
        primary_stats = {
            'rx_packets': int(rx_packets) if rx_packets is not None else 0,
            'rx_bytes': int(rx_bytes) if rx_bytes is not None else 0,
            'tx_packets': int(tx_packets) if tx_packets is not None else 0,
            'tx_bytes': int(tx_bytes) if tx_bytes is not None else 0,
        }
        intf_statistics[intf] = {
            'name': name,
            'mac': mac,
            'ip_address': ip_address,
            'state': state,
            'speed': speed,
            'duplex': duplex,
            'primary_stats': primary_stats,
        }
    return intf_statistics


def get_secondary_interface_data(intf_statistics):
    for intf, stats in intf_statistics.items():
        statistics_output = get_statistics_output(intf)
        if intf.startswith(('vti', 'tun')):
            rx_packets2, rx_bytes2, tx_packets2, tx_bytes2 = get_intf_stats_vpn_vlan(intf)
        else:
            rx_packets2, rx_bytes2, tx_packets2, tx_bytes2 = extract_statistics(statistics_output)

        # Store secondary statistics in dictionary
        secondary_stats = {
            'rx_packets': int(rx_packets2) if rx_packets2 is not None else 0,
            'rx_bytes': int(rx_bytes2) if rx_bytes2 is not None else 0,
            'tx_packets': int(tx_packets2) if tx_packets2 is not None else 0,
            'tx_bytes': int(tx_bytes2) if tx_bytes2 is not None else 0,
        }
        stats['secondary_stats'] = secondary_stats

    return intf_statistics


def get_statistics_output(intf):
    if intf.startswith('data'):
        intf_mode = get_intf_mode(intf)
        if intf_mode == 'virtio':
            stat_interface = f'host-{intf}'
        else:
            stat_interface = intf
    elif any(intf.startswith(prefix) for prefix in ['tun', 'wifi', 'lte', 'vti', 'tap', 'VirtualEthernet']):
        stat_interface = intf
    else:
        stat_interface = vpp_if.get_tap_interface_by_host_ifname(intf).get('ifname')


    try:
        result = subprocess.run(['sudo', '/usr/bin/vppctl', 'show', 'interface', stat_interface],
                                capture_output=True)
        statistics_output = result.stdout

        if not isinstance(statistics_output, str):
            statistics_output = statistics_output.decode()

        return statistics_output.strip()
    except subprocess.CalledProcessError as e:

        syslog.syslog(syslog.LOG_ERR, f"Command failed with return code {e.returncode}")
        return None


def extract_statistics(output):
    regex_patterns = {
        'rx_packets': r"rx packets\s+(\d+)",
        'rx_bytes': r"rx bytes\s+(\d+)",
        'tx_packets': r"tx packets\s+(\d+)",
        'tx_bytes': r"tx bytes\s+(\d+)"
    }

    matches = {key: re.search(pattern, output) for key, pattern in regex_patterns.items()}
    extracted_values = {key: match.group(1) if match else None for key, match in matches.items()}

    return tuple(extracted_values.values())


def extract_value(pattern, output):
    match = re.search(pattern, output)
    return match.group(1) if match else None


def get_interface_data(intfs):

    intf_statistics = {}

    for intf in intfs:
        name = intf
        statistics_output = get_statistics_output(intf)
        ip_address = get_br_data_ip(intf)

        # When the physical interfaces are in dpdk mode - obtain their state, speed and duplex via vppctl
        iface = str(intf).split('.')[0]# ethernet and VLAN case
        intf_mode = get_intf_mode(iface)
        if intf.startswith('data') and intf_mode == 'dpdk':
            hard_output = subprocess.check_output(shlex.split(VPP_SH_HARD_COMMAND + iface)).decode(
                'utf-8')
            state_pattern = r'carrier (\w+)'
            speed_pattern = r'Link speed: (\d+ Mbps)'
            duplex_pattern = r'(\w+)\s+duplex'
            mac_pattern = r'Ethernet address\s+(\w{2}(?::\w{2}){5})'
            state = extract_value(state_pattern, hard_output)
            speed = extract_value(speed_pattern, hard_output)
            if not speed:
                speed_pattern = r'Link speed: (\d+ Gbps)'
                speed = extract_value(speed_pattern, hard_output)
                if speed:
                    speed = str(speed).replace(' ','')
            if not speed:
                speed = "unknown"
            duplex = extract_value(duplex_pattern, hard_output)
            if not duplex:
                duplex = "unknown"
            mac = extract_value(mac_pattern, hard_output)

        else:
            state = get_intf_state(statistics_output, intf)
            speed = get_intf_speed(intf)
            duplex = get_intf_duplex(intf)
            mac = get_mac_address(iface)

        if intf.startswith(('vti', 'tun')):
            rx_packets, rx_bytes, tx_packets, tx_bytes = get_intf_stats_vpn_vlan(intf)
        else:
            rx_packets, rx_bytes, tx_packets, tx_bytes = extract_statistics(statistics_output)

        # Store primary statistics in dictionary
        primary_stats = {
            'rx_packets': int(rx_packets) if rx_packets is not None else 0,
            'rx_bytes': int(rx_bytes) if rx_bytes is not None else 0,
            'tx_packets': int(tx_packets) if tx_packets is not None else 0,
            'tx_bytes': int(tx_bytes) if tx_bytes is not None else 0,
        }
        intf_statistics[intf] = {
            'name': name,
            'mac': mac,
            'ip_address': ip_address,
            'state': state,
            'speed': speed,
            'duplex': duplex,
            'primary_stats': primary_stats,
        }

        # Sleep for 7 seconds
    sleep(7)

    # Collect secondary statistics for all interfaces
    for intf, stats in intf_statistics.items():
        statistics_output = get_statistics_output(intf)
        if intf.startswith(('vti', 'tun')):
            rx_packets2, rx_bytes2, tx_packets2, tx_bytes2 = get_intf_stats_vpn_vlan(intf)
        else:
            rx_packets2, rx_bytes2, tx_packets2, tx_bytes2 = extract_statistics(statistics_output)

        # Store secondary statistics in dictionary
        secondary_stats = {
            'rx_packets': int(rx_packets2) if rx_packets2 is not None else 0,
            'rx_bytes': int(rx_bytes2) if rx_bytes2 is not None else 0,
            'tx_packets': int(tx_packets2) if tx_packets2 is not None else 0,
            'tx_bytes': int(tx_bytes2) if tx_bytes2 is not None else 0,
        }
        stats['secondary_stats'] = secondary_stats

    return intf_statistics


def display_data(intf_type, command_name, formatting="normal"):
    interfaces = []
    if intf_type == 'tap':
        cmd_process = subprocess.Popen(VPP_SH_TAP_COMMAND, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = cmd_process.communicate()
        if cmd_process.returncode != 0:
            error_msg = "Error: " + err.decode("utf-8")
            syslog.syslog(syslog.LOG_ERR, error_msg)
        else:
            # Process the output
            interfaces = out.decode("utf-8").strip().split("\n")
    elif intf_type == 'VirtualEthernet':
        cmd_process = subprocess.Popen(VPP_SH_VTI_COMMAND, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = cmd_process.communicate()
        if cmd_process.returncode != 0:
            error_msg = "Error: " + err.decode("utf-8")
            syslog.syslog(syslog.LOG_ERR, error_msg)
        else:
            # Process the output
            interfaces = out.decode("utf-8").strip().split("\n")
    elif intf_type == 'data':
        just_data_interfaces = [name for name in all_interfaces if name.startswith(f'{intf_type}')]
        interfaces = [name for name in just_data_interfaces if '.' not in name]
    else:
        interfaces = [name for name in all_interfaces if name.startswith(f'{intf_type}')]
    try:
        statistics = get_interface_data(interfaces)
        print_statistics(statistics, formatting, command_name)
    except AttributeError:
        syslog.syslog(syslog.LOG_ERR, f'There is no {command_name}!')


def display_interface_data(intf_t, cmd, mssg, formatting):
    if mssg and formatting != 'csv':
        print('-' * 79)
        print(mssg)
    for interface_type in intf_t:
        display_data(interface_type, cmd, formatting)


def format_label(label, max_width):
    return (label[:2].upper() + label[2:].replace('_', ' ')).capitalize() + ':'.ljust(max_width - len(label))


def print_normal_statistics(stats, cmd):
    print("-" * 79)
    for label in LABELS:
        if 'vlan' == cmd and label not in ['name', 'mac']:
            continue
        value = stats.get(label)
        if value is not None:
            label_formatted = format_label(label, MAX_LABEL_WIDTH)
            value_formatted = f"{value:<{MAX_VALUE_WIDTH}}"
            if label == "mac":
                print(label_formatted.upper(), value_formatted)
            elif label in ["ip_address", "duplex", "speed"]:
                if value:
                    print(label_formatted, value_formatted)
            else:
                print(label_formatted, value_formatted)

    for label in PACKET_LABELS:
        primary_value = stats['primary_stats'].get(label.replace(" ", "_"))
        secondary_value = stats['secondary_stats'].get(label.replace(" ", "_"))

        if primary_value is not None and secondary_value is not None:
            value_diff = int((secondary_value - primary_value) / 7)
            value_formatted = f"{value_diff:<{MAX_VALUE_WIDTH}}"
            label_formatted = format_label(label + ' Rate', MAX_LABEL_WIDTH)
            print(label_formatted, value_formatted)


def print_csv_statistics(stats):

    speed = stats.get('speed', '')
    duplex = stats.get('duplex', '')

    data = [
        stats['name'], stats['mac'], stats['ip_address'], stats['state'], speed, duplex,
        *[int((int(stats['secondary_stats'][key]) - int(stats['primary_stats'][key])) / 7) for key in
          ['rx_packets', 'rx_bytes', 'tx_packets', 'tx_bytes']]
    ]
    print(','.join(map(str, [d for d in data if d != ''])))


def format_label_per_statistics(label):
    return label.replace('_', ' ').capitalize()


def print_statistics(intf_statistics, formatting='normal', cmd=None):
    if formatting == "normal":
        for interface, stats in intf_statistics.items():
            print_normal_statistics(stats, cmd)
    elif formatting == "csv":
        for interface, stats in intf_statistics.items():
            print_csv_statistics(stats)


if __name__ == "__main__":
    signal.signal(signal.SIGINT, signal_handler)
    argc = len(sys.argv)
    all_interfaces = get_all_intf_names()

    if argc < 2:
        syslog.syslog(syslog.LOG_ERR, "Not enough arguments. Exit!")
        sys.exit(1)

    command_map = {
        'interfaces': (None, 'All interfaces:'),
        'physical': (['data'], 'Physical interfaces:'),
        'nat': (['nat'], 'NAT interfaces:'),
        'mgmt': (['mgmt'], 'Management interfaces:'),
        'loopback': (['lo'], 'Loopback interfaces:'),
        'vpn': (['tun', 'vti', 'wg'], 'TUN interfaces:'),
        'virtual': (['tap', 'VirtualEthernet'], 'Virtual interfaces:'),
        'wifi': (['wifi'], 'WiFi interfaces:'),
        'lte': (['lte'], 'LTE interfaces:'),
        'vlan': (None, 'VLAN interfaces:'),
    }

    command = sys.argv[1]
    output_format = "csv" if argc > 2 and sys.argv[2] == "csv" else "normal"

    if command in command_map:
        interface_types, message = command_map[command]
        if command == 'vlan':
            # Handle VLAN interfaces separately using regex
            vlan_pattern = re.compile(r'^data\d+\.\d+$')
            data_interfaces = [name for name in all_interfaces if vlan_pattern.match(name)]
            try:
                display_interface_data(data_interfaces, command, message, output_format)
            except AttributeError:
                syslog.syslog(syslog.LOG_ERR, f"Unknown command:{command}!")

        elif interface_types is None:
            all_interface_types = []
            for intf_types, _ in command_map.values():
                if intf_types is not None:
                    all_interface_types += intf_types
            interface_types = all_interface_types

            data_interfaces = []
            if interface_types == 'tap':
                process = subprocess.Popen(VPP_SH_TAP_COMMAND, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                output, error = process.communicate()
                if process.returncode != 0:
                    error_message = "Error: " + error.decode("utf-8")
                    syslog.syslog(syslog.LOG_ERR, error_message)
                else:
                    # Process the output
                    data_interfaces = output.decode("utf-8").strip().split("\n")
            elif interface_types == 'VirtualEthernet':
                process = subprocess.Popen(VPP_SH_VTI_COMMAND, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                output, error = process.communicate()
                if process.returncode != 0:
                    error_message = "Error: " + error.decode("utf-8")
                    syslog.syslog(syslog.LOG_ERR, error_message)
                else:
                    # Process the output
                    data_interfaces = output.decode("utf-8").strip().split("\n")
            else:
                data_interfaces = [name for name in all_interfaces if name.startswith(f'{interface_types}')]
            try:
                initial_statistics = get_initial_interface_data(all_interfaces)
                sleep(7)
                final_statistics = get_secondary_interface_data(initial_statistics)

                if message and output_format != 'csv':
                    print('-' * 79)
                    print(message)
                print_statistics(final_statistics, output_format)
            except AttributeError:
                syslog.syslog(syslog.LOG_ERR, f"Unknown command:{command}!")
            sys.exit(0)
        else:
            try:
                display_interface_data(interface_types, command, message, output_format)
            except AttributeError:
                syslog.syslog(syslog.LOG_ERR, f"Unknown command:{command}!")
