#!/usr/bin/env python3

import subprocess
import os
import sys
import time
import xml.etree.ElementTree as ET
import logging
import signal

# Configure logging
logging.basicConfig(filename='trace.log', level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

USER_FILES = "/home/admin/user_files/"


def signal_handler(_, __):
    print('\nTerminated by the user!')
    sys.exit(1)  # Exit the program with a status code of 1


signal.signal(signal.SIGINT, signal_handler)


def trace_status_active():
    try:
        status_output = subprocess.check_output(['sudo', '/usr/bin/vppctl', 'pcap', 'trace', 'status']).decode()
        if "pcap dispatch capture already disabled" in status_output:
            return False
        elif "dispatch trace already enabled" in status_output:
            return True
        elif "pcap rx and tx dispatch capture enabled" in status_output:
            return True
    except subprocess.CalledProcessError:
        return False
    return False


def stop_trace(file=None):
    subprocess.run(['sudo', '/usr/bin/vppctl', 'pcap', 'trace', 'off'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if file is not None:
        sys.stdout.write(f'\nTcpdump output is stored in {USER_FILES}{file}\n')


def move_pcap_file(filename=None):
    if filename:
        logging.info(f"This is the filename {filename}")
        source_file = os.path.join("/tmp", filename)
        logging.info(f"This is the {source_file}")
        if os.path.exists(source_file):
            subprocess.run(['sudo', 'mv', source_file, USER_FILES])
            logging.info(f"Moved {filename} to {USER_FILES}")
        else:
            logging.error(f"File {filename} does not exist in /tmp/")
    else:
        source_file = "/tmp/rxtx.pcap"
        if os.path.exists(source_file):
            subprocess.run(['sudo', 'tcpdump', '-r', f'{source_file}'])
            # subprocess.run(['sudo', 'rm', f'{source_file}'])
            logging.info(f"Deleted and printed content of {source_file}")
        else:
            logging.error("Default file rxtx.pcap does not exist in /tmp/")


def execute_trace(interface, time_in_seconds, filename=None):
    start_time = time.time()
    # Stop the pcap trace
    if trace_status_active():
        stop_trace()

    if filename:
        subprocess.run(['sudo', '/usr/bin/vppctl', 'pcap', 'trace', 'rx', 'tx', 'file', filename, 'intfc', interface,
                        'max', '999999999'])
    else:
        subprocess.run(['sudo', '/usr/bin/vppctl', 'pcap', 'trace', 'rx', 'tx', 'intfc', interface, 'max', '999999999'])

    # Continuously check trace status and time interval
    while time.time() - start_time < time_in_seconds:
        message = "Capturing packets"
        chars = ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧']
        i = (int(time.time() * 10)) % len(chars)
        sys.stdout.write('\r' + chars[i] + chars[i] + chars[i] + chars[i] + chars[i] + ' ' + message +
                         ' ' + chars[i] + chars[i] + chars[i] + chars[i] + chars[i])
        sys.stdout.flush()
        time.sleep(0.5)

    sys.stdout.write('\r' + ' ' * 50 + '\r')
    sys.stdout.flush()

    if filename:
        stop_trace(filename)
    else:
        stop_trace()


if __name__ == "__main__":
    interface = ""
    filename = ""
    time_in_seconds = 10
    args = sys.argv[1:]
    ifmode = ""

    while args:
        if args[0] == '-i':
            interface = args[1]
        elif args[0] == '-t':
            time_in_seconds = int(args[1])
        elif args[0] == '-f':
            filename = args[1]
        args = args[2:]

    if not os.path.isdir(USER_FILES):
        os.mkdir(USER_FILES)
        os.chmod(USER_FILES, 0o755)

    '''
        Get the mode of the interface in order to define the port
        Example: if dpdk -> data0 , if virtio -> host-data0
    '''

    try:
        output = subprocess.check_output(['sysrepocfg', '--datastore=running', '--format=xml',
                                          f"--xpath=/ucpe:config/interfaces:interface[name='{interface}']/mode",
                                          '--export']).decode().strip()
        root = ET.fromstring(output)
        namespaces = {'ucpe': 'urn:ucpe', 'interfaces': 'urn:interfaces'}
        ifmode = root.find(".//{urn:interfaces}mode", namespaces).text.strip()

    except Exception as e:
        logging.error(f"The interface mode can not be obtained. Error: {e}.")

    intf = interface
    if ifmode == 'virtio':
        intf = f'host-{interface}'

    execute_trace(intf, time_in_seconds, filename)
    move_pcap_file(filename)
