#! /usr/bin/env python3
# -*- coding: utf-8 -*-
import os
from subprocess import PIPE, Popen, TimeoutExpired
import sys
import re
import logging, logging.handlers
import time
import shutil
import json
import requests
from urllib.parse import urlparse, parse_qs
import ftplib
import netifaces

log = None

TEMPORARY_DIR = '/tmp/'
SUPPORTED_IMAGE_FORMATS = ['qcow2', 'iso', 'raw', 'img', 'tar']
LOCK_FILE = TEMPORARY_DIR + 'registry_api.lock'
BIND_INTERFACE_FILE = '/kubernetes/bind-interface-file'
DOCKERFILE = '''
# Shared Volume
FROM scratch
ADD {} /disk/
'''
DOCKER_FILENAME = TEMPORARY_DIR + 'Dockerfile'
DOCKER_IMAGE = TEMPORARY_DIR + 'output.tar'
CLUSTER_FILE='/config/cluster.json'
HA_MODE_FILE='/config/.hamode'
KUBERNETES_VNF_DIR='/kubernetes/vnfs/'
KUBERNETES_VNF_STATUS_FILE=KUBERNETES_VNF_DIR+'vnfs.json'


def set_log(name, level):
    global log
    log = logging.getLogger(name=name)
    log.setLevel(level=level)

    syslog_hnd = logging.handlers.SysLogHandler(address='/dev/log')
    formatter = logging.Formatter('%(name)s: %(levelname)s %(message)s')
    syslog_hnd.setFormatter(formatter)
    log.addHandler(syslog_hnd)


def set_status(status, image):
    """
    Write status to /tmp/registry_api_<image_name>
    """
    log.debug(f'Image \'{image}\' is in \'{status}\' state.')
    image = re.sub('[/:]', '_', image)
    with open('/tmp/registry_api_' + image, 'w') as f:
        print(status, file=f, end='')


def locks_and_key():
    """
    Don't call registry api (lock it) with more than one operation.
    With this implementation, if we have bulk upload of images (2 or more in same time)
    every process of registry_api.py will wait previosly started process.
    """
    while True:
        if os.path.exists(LOCK_FILE):
            # Delete LOCK_FILE older than 5 minutes
            try:
                diffs = int(time.time()) - int(os.path.getmtime(LOCK_FILE))
            except Exception as e:
                diffs = 0
            if diffs > 300:
                os.remove(LOCK_FILE)
        else:
            fd = os.open(LOCK_FILE, os.O_RDWR | os.O_CREAT)
            os.fdopen(fd, "w+").close()
            time.sleep(5)
            return


def write_json_file(json_dict, target_file):
    """
    Writes a dictionary to a file.
    :json_dict - The dictionary to be dumped to the file
    :target_file - The file to be written
    """
    with open(target_file, 'w') as content_file:
        json_contents = json.dumps(json_dict,  sort_keys=True,
                 indent=4, separators=(',', ': '))
        content_file.write(json_contents)


def get_json_dict_from_file(file):
    """
    Returns a dictionary object, constructed from the JSON String in file
    :file - A file containing valid JSON
    """
    with open(file, 'r') as content_file:
        json_content = content_file.read().rstrip()
        return json.loads(json_content)


def execute_shell_command(command, timeout=None, quiet=False):
    """
    Execute shell command with Popen.
    Args:
    - timeout - set max timeout for command. Default is None.
    """
    call = Popen(command, shell=True,
                            encoding='utf-8', text=True,
                            stdout=PIPE,
                            stderr=PIPE)
    stdoutdata, stderdata = "", f"Timeout expires while waiting to execute {command} with '{timeout}' timeout."
    try:
        stdoutdata, stderdata = call.communicate(timeout=timeout)
    except TimeoutExpired:
        call.kill()
    if call.returncode != 0 and not quiet:
        sys.exit(stderdata)
    else:
        return stdoutdata


def delete_garbage(name, tag, on_error=False, filename=''):
    """
    Delete unnecessary files and Docker images
    """
    repo_dir = '/var/lib/local-registry/docker/registry/v2/repositories/' + name
    file_path = os.path.join(TEMPORARY_DIR, filename.split('/')[-1])

    try:
        os.remove(file_path) if os.path.exists(file_path) and file_path != TEMPORARY_DIR else None
        os.remove(DOCKER_FILENAME) if os.path.exists(DOCKER_FILENAME) else None
        os.remove(DOCKER_IMAGE) if os.path.exists(DOCKER_IMAGE) else None
        shutil.rmtree(repo_dir) if on_error and os.path.exists(repo_dir) else None
        os.remove(LOCK_FILE) if os.path.exists(LOCK_FILE) else None
        # Remove images with <none> tag
        images = json.loads(execute_shell_command('crictl images -o json'))
        for img in images['images']:
            if img['repoTags'] == []:
                execute_shell_command(f'ctr -n k8s.io images delete {img["id"]}')
    except Exception as e:
        # Delete lock file if we have problem
        os.remove(LOCK_FILE) if os.path.exists(LOCK_FILE) else None
        sys.exit(e)


def import_from_remote(name, tag, registry_url=None, username=None, password=None):
    """
    Import image from remote repository, like dockerhub.
    """
    image = name + ':' + tag
    registry_url = None if registry_url == "None" else registry_url
    username, password = (None, None) if username == "None" or password == "None" else (username, password)
    bind_interface = 'br-data0'
    try:        
        if registry_url:
            registry_url = registry_url+'/library' if '/' not in name else registry_url
            remote_image = f'{registry_url.rstrip("/")}/{name}:{tag}'
        else:
            remote_image = f'docker.io/library/{name}:{tag}' if '/' not in name else f'docker.io/{name}:{tag}'
        
        if os.path.isfile(BIND_INTERFACE_FILE) and os.stat(BIND_INTERFACE_FILE).st_size != 0:
            with open(BIND_INTERFACE_FILE) as f:
                bind_interface = f.read()

        ip = netifaces.ifaddresses(bind_interface)[netifaces.AF_INET][0]['addr']
        repo = ip + ':5001/' + image

        # 1. Pull image from remote repo
        set_status('pulling', image)

        if username and password:
            execute_shell_command(f'ctr -n k8s.io images pull --platform linux/amd64 --user {username}:{password} {remote_image}')
        else:
            execute_shell_command(f'ctr -n k8s.io images pull --platform linux/amd64 {remote_image}')           

        # # 2. Get the image tag
        # loaded_image = None
        # images = client.images()

        # if registry_url:
        #     for img in images:
        #         if img.get("RepoTags"):
        #             if f"{name}:{tag}" in img["RepoTags"] or f"{repo_without_scheme}:{tag}" in img["RepoTags"]:
        #                 loaded_image = img["Id"].split(':')[1][0:12]
        #                 break
        #         else:
        #             print("RepoTags not found for image:", img)

        # else:
        #     loaded_image = client.inspect_image(image)['Id'].split(':')[1][0:12]


        # if not loaded_image:
        #     raise Exception(f"Image {name}:{tag} not found")

        # # 3. Set the tag to remote if we have url
        # if registry_url:
        #     repo = getoutput(
        #         "cat /etc/hostname") + ':30501/' + name
        # client.tag(loaded_image, repo, tag=tag, force=True)

        # # 4. Push the image in local repo
        # set_status('pushing', image)
        # 2. Tag image and push it to registry
        set_status('pushing', image)
        execute_shell_command(f'ctr -n k8s.io images push --plain-http --platform linux/amd64 --allow-non-distributable-blobs {repo} {remote_image}')
        # 3. Remove uploaded image from containerd
        execute_shell_command(f'ctr -n k8s.io images delete {remote_image}')
        set_status('ready', image)

    except Exception as e:
        log.error(e)
        set_status('error', image)
        sys.exit(e)


def upload(name, tag, filename):
    """
    Upload image from web, FTP server or via API.
    """
    file_extension = filename.lower().split('.')[-1]

    parsed_source = urlparse(filename)
    query_string = parse_qs(parsed_source.query)
    image = name + ':' + tag

    if parsed_source.scheme in ['http', 'https', 'ftp']:
        # Handle case when a URL is provided
        source = filename
        filename = query_string.get('files')[0] if 'files' in query_string else os.path.basename(parsed_source.path)
        local_file = os.path.join(TEMPORARY_DIR, filename)
        set_status('downloading', image)
        if parsed_source.scheme in ['http', 'https']:
            response = requests.get(source, verify=False, timeout=60) 
            with open(local_file, 'wb') as f:
                f.write(response.content)
        elif parsed_source.scheme == 'ftp':
            ftp = ftplib.FTP()
            ftp.connect(parsed_source.hostname, parsed_source.port or 21)
            ftp.login(parsed_source.username or 'anonymous', parsed_source.password or '')
            ftp.cwd(os.path.dirname(parsed_source.path))

            with open(local_file, 'wb') as f:
                ftp.retrbinary('RETR ' + os.path.basename(parsed_source.path), f.write)

            ftp.quit()

        file_path = local_file
    else:
        file_path = os.path.join(TEMPORARY_DIR, filename)

    try:
        bind_interface = 'br-data0'
        if os.path.isfile(BIND_INTERFACE_FILE) and os.stat(BIND_INTERFACE_FILE).st_size != 0:
            with open(BIND_INTERFACE_FILE) as f:
                bind_interface = f.read()

        ip = netifaces.ifaddresses(bind_interface)[netifaces.AF_INET][0]['addr']
        if os.path.exists(CLUSTER_FILE) and os.path.exists(HA_MODE_FILE):
            with open(CLUSTER_FILE) as config_file:
                data = json.load(config_file)
            repo = data['virtual_address'] + ':5001/' + image
        repo = ip + ':5001/' + image
        if file_extension == 'tar':
            # 1. Import from .tar
            stdout = execute_shell_command(f'ctr -n k8s.io image import --digests --skip-digest-for-named {file_path}')
            loaded_image = stdout.split(' ')[1]
            # 2. Tag image and push it to registry (--platform linux/amd64 parameter is removed because amd64 is default)
            set_status('pushing', image)
            execute_shell_command(f'ctr -n k8s.io images push --plain-http --allow-non-distributable-blobs {repo} {loaded_image}')
            # 3. Remove uploaded image from containerd
            execute_shell_command(f'ctr -n k8s.io images delete {loaded_image}')
        elif file_extension in SUPPORTED_IMAGE_FORMATS:
            vnf_file = f'{name}:{tag}'
            vnf_image = KUBERNETES_VNF_DIR + vnf_file
            vnf_data = get_json_dict_from_file(KUBERNETES_VNF_STATUS_FILE)
            vnf_data['images'].append({'name': name, 'tag': tag, 'file': vnf_file})
            # Remove duplicates
            unique_data = list({json.dumps(item, sort_keys=True) for item in vnf_data['images']})
            vnf_data['images'] = [json.loads(item) for item in unique_data]
            write_json_file(vnf_data, KUBERNETES_VNF_STATUS_FILE)
            shutil.move(TEMPORARY_DIR+filename, vnf_image)
        set_status('ready', image)
    except Exception as e:
        sys.exit(e)


if __name__ == "__main__":
    set_log('registry-api', logging.DEBUG)
    if len(sys.argv) < 3:
        sys.exit("Missing config params!")
    else:
        image = sys.argv[1] + ':' + sys.argv[2]
        locks_and_key()

        name = sys.argv[1]
        tag = sys.argv[2]
        registry_url = None
        username = None
        password = None

        try:
            # Create status file
            status_file = '/tmp/registry_api_{}'.format(re.sub('[/:]', '_', image))
            if not os.path.exists(status_file):
                fd = os.open(status_file, os.O_RDWR|os.O_CREAT)
                os.fdopen(fd, "w+").close()

            if sys.argv[3].lower().endswith(tuple(SUPPORTED_IMAGE_FORMATS)):
                upload(name, tag, sys.argv[3])
                delete_garbage(sys.argv[1], sys.argv[2], False, sys.argv[3])
            else:
                registry_url = sys.argv[3]

                if len(sys.argv) >= 5:
                    username = sys.argv[4]

                if len(sys.argv) >= 6:
                    password = sys.argv[5]

                if registry_url and username and password is not None:
                    import_from_remote(name, tag, registry_url, username, password)
                    delete_garbage(sys.argv[1], sys.argv[2], False)
                elif len(sys.argv) >= 3:
                    import_from_remote(name, tag, registry_url)
                    delete_garbage(sys.argv[1], sys.argv[2], False)
        except SystemExit as e:
            set_status('error', image)
            delete_garbage(sys.argv[1], sys.argv[2], True, sys.argv[3])
            sys.exit(f'Registry API Error: {e}')
        except Exception as e:
            set_status('error', image)
            delete_garbage(sys.argv[1], sys.argv[2], True, sys.argv[3])
            sys.exit(f'Registry API Error: {e}')

    sys.exit(0)
