#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys
import os
import ipaddress

import ucpeif.wg_concentratorif as wgc
import edgility_db.edgility_db as db
import mysql.connector
from mysql.connector import errorcode

STATIC_POOL_PROPERTIES = (
            "hostname VARCHAR(255) UNIQUE, "
            "tunnel_ip VARCHAR(255)"
        )
STATIC_POOL_COLUMNS = [
            "hostname",
            "tunnel_ip"
    ]
STATIC_POOL_NAME = 'static_clients_'
DYNAMIC_POOL_NAME = 'dynamic_clients_'
DYNAMIC_POOL_COLUMNS = [
            "public_key",
            "hostname",
            "peer_endpoint",
            "tunnel_ip",
            "allowed_ips",
            "password"
    ]

DNSMASQ_HOSTS_DIR = '/config/dnsmasq/hostsdir'


def get_db_api():
    try:
        db_int = db.EdgilityDatabase()
    except:
        print("Wireguard database is currently busy, please try again later.")
        exit(1)
    return db_int

def db_table_name(db_pool, intf_name):
        return db_pool + intf_name

def set_wireguard_concentrator(hostname, password, public_key, conc_port, endpoint):
    client = wgc.WireGuardConcentratorClient(hostname, password, public_key, int(conc_port), endpoint)
    print(client)
   
def add_del_static_client_in_db(intf_name, hostname, tunnel_ip=None, delete=False):
    db_int = get_db_api()
    
    if delete:
        db_int.delete_value_from_table(db_table_name(STATIC_POOL_NAME, intf_name), 'hostname', hostname)
    else:
        tunnel_ip_int = int(ipaddress.IPv4Address(tunnel_ip))
        static_values = (str(hostname), tunnel_ip_int)
        db_int.create_table(db_table_name(STATIC_POOL_NAME, intf_name), STATIC_POOL_PROPERTIES)

        if verify_if_entry_exists_in_table(db_table_name(STATIC_POOL_NAME, intf_name), 'tunnel_ip', tunnel_ip_int) or verify_if_entry_exists_in_table(db_table_name(DYNAMIC_POOL_NAME, intf_name), 'tunnel_ip', tunnel_ip_int):
            print(f"Client with tunnel ip {tunnel_ip} already exists. Please use another one.")
            return

        db_int.insert_values_if_not_exist_otherwise_update(db_table_name(STATIC_POOL_NAME, intf_name), STATIC_POOL_COLUMNS, static_values,'tunnel_ip', tunnel_ip_int)
        try:
            old_client_data = db_int.fetch_row_by_param(db_table_name(DYNAMIC_POOL_NAME, intf_name), 'hostname', hostname)
            pub_key, peer_e_point, all_ips, passw = old_client_data[0], old_client_data[2], old_client_data[4], old_client_data[5]
            dynamic_values = (str(pub_key), str(hostname), str(peer_e_point), tunnel_ip_int, str(all_ips), str(passw))
            db_int.insert_values_if_not_exist_otherwise_update(db_table_name(DYNAMIC_POOL_NAME, intf_name), DYNAMIC_POOL_COLUMNS, dynamic_values,'tunnel_ip', tunnel_ip_int)
        except Exception as e:
            pass

def verify_if_entry_exists_in_table(table, param, value):
    db_int = get_db_api()
    entry = db_int.verify_if_entry_exists_in_table_by_param(table, param, value)
    if entry:
        return True

def set_domain_lookup(hostname, d_lookup):
    db_int = get_db_api()
    
    tables = db_int.get_all_tables('wireguard')
    for table in tables:
        if table.startswith('dynamic') or table.startswith('static'):
            if db_int.verify_if_entry_exists_in_table_by_param(table, 'hostname', hostname):
                create_dns_hosts_dir()
                ip_int = db_int.fetch_from_table_by_param(table, 'tunnel_ip', 'hostname', hostname)
                ip_str = convert_ip_int_to_string(ip_int)

                if d_lookup == True or d_lookup == 'True':
                    create_hosts_file(ip_str, hostname)
                    # restart dnsmasq for now due to sporadic failure in resolving on new entry
                    os.system('systemctl restart dnsmasq.service')
                else:
                    delete_hosts_file(ip_str)

def convert_ip_int_to_string(ip_int):
        return str(ipaddress.ip_address(int(ip_int)))
    
def create_dns_hosts_dir():
    if not os.path.exists(DNSMASQ_HOSTS_DIR):
        os.makedirs(DNSMASQ_HOSTS_DIR)
        os.system('systemctl restart dnsmasq.service')

def create_hosts_file(ip, hostname):
    file_name = os.path.join(DNSMASQ_HOSTS_DIR, ip)
    file_content = f"{ip} {hostname}\n"

    try:
        with open(file_name, 'w') as file:
            file.write(file_content)
        print(f"File '{file_name}' created successfully.")
    except IOError as e:
        print(f"Error creating file '{file_name}': {e}")
        
def delete_hosts_file(ip):
    file_name = os.path.join(DNSMASQ_HOSTS_DIR, ip)

    try:
        if os.path.exists(file_name):
            os.remove(file_name)
            print(f"File '{file_name}' deleted successfully.")
            os.system('systemctl restart dnsmasq.service')
        else:
            print(f"File '{file_name}' does not exist.")
    except OSError as e:
        print(f"Error deleting file '{file_name}': {e}")

if __name__ == "__main__":
    argc = len(sys.argv)
    if argc < 3:
        sys.exit(1)

    if sys.argv[1] == "dynamic":
        set_wireguard_concentrator(sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6])
    elif sys.argv[1] == "static":
        if sys.argv[4] == "delete":
            add_del_static_client_in_db(sys.argv[2], sys.argv[3], delete=True)
        else:
            add_del_static_client_in_db(sys.argv[2], sys.argv[3], sys.argv[4])
    elif sys.argv[1] == "domain_lookup":
        set_domain_lookup(sys.argv[2], sys.argv[3])

    sys.exit(0)

