#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import threading
import time
import signal
import sys
import logging
import ipaddress
from systemd.journal import JournalHandler

import edgility_db.edgility_db as db
import vppapi_if.vppapi_if as vppif


TIMESTAMP_POOL_NAME = 'timestamp'

log = logging.getLogger('wg-conc-thread')
log.addHandler(JournalHandler())
log.setLevel(logging.INFO)

class HostCleaner(threading.Thread):
    def __init__(self, log_level=logging.ERROR):
        super().__init__()
        self.stop_event = threading.Event()

    def run(self):
        while not self.stop_event.is_set():
            self.clean_hosts()
            time.sleep(60)  # Check every minute
                
    def clean_hosts(self):
        db_int = get_db_api()
        if db_int:
            try:
                # Fetch all rows from the 'timestamp' table
                timestamp_table = db_int.fetch_all_rows_from_table(TIMESTAMP_POOL_NAME)
                for row in timestamp_table:
                    wg_key, date, updated_date = row
                    # Convert updated_date to timestamp
                    updated_timestamp = int(time.mktime(updated_date.timetuple()))
                    # Check if the date is no older than 15 minutes
                    if time.time() - updated_timestamp > 900:
                        vpp = get_vpp_client()
                        peer = vpp.wg_get_peer_by_public_key(wg_key.strip())
                        if peer: 
                            allowed_ips = peer['allowed_ips']
                            if_index = peer['wgif']
                            peer_idx = peer['peer_idx']
                        
                            for a_ip in allowed_ips:
                                ip_addr = ipaddress.IPv4Interface(str(a_ip))
                                ip_netmask = ip_addr.network.prefixlen

                                ret = vpp.del_ip_route('{}/{}' .format(ip_addr.ip, ip_netmask), next_hop_sw_if_index=if_index)
                                if ret:
                                    log.error('Del ip route fail route {}/{} via {} ret={}'.format(ip_addr.ip, ip_netmask, if_index ,ret))

                                ret = vpp.wg_del_peer(peer_idx=peer_idx)
                                del(vpp)
                                if ret:
                                    self.logger.error("Error deleting peer {}" .format(wg_key))
                            
                            tables = db_int.get_all_tables('wireguard')
                            for table in tables:
                                try:
                                    db_int.delete_value_from_table(table, 'public_key', wg_key)
                                except Exception as e:
                                    log.error(f"Host already deleted")
                                log.error(f"Host with key {wg_key} has not been active for more than 15 minutes. Detaching...")
                        else:
                            db_int.delete_value_from_table(table, 'public_key', wg_key)
                            log.error(f"Tunnel for peer with key {wg_key} does not exsists!")
                    else:
                        log.error(f"Host with key {wg_key} is still active")
            except Exception as e:
                log.error(e)

def get_db_api():
    try:
        db_int = db.EdgilityDatabase()
        return db_int
    except Exception as e:
        log.error(f"Error connecting to the database: {e}")
        return None

def get_vpp_client():
    return vppif.VppApiInterface('wireguard_concentrator_timestamp_thread')

def sig_handler(signum, frame):
    # Handle termination signal (SIGTERM)
    log.error("Received SIGTERM, stopping host cleaner...")
    sys.exit(0)

def main():
    # Register signal handler for SIGTERM
    signal.signal(signal.SIGTERM, sig_handler)

    # Start the host cleaner thread
    cleaner = HostCleaner()
    cleaner.start()

    # Wait for cleaner to finish
    cleaner.join()

if __name__ == "__main__":
    main()
