#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import threading
import sys
import time
import signal
import logging

from systemd.journal import JournalHandler
import edgility_db.edgility_db as db

TIMESTAMP_POOL_COLUMNS = [
            "public_key",
            "date"
    ]
TIMESTAMP_POOL_NAME = 'timestamp'

log = logging.getLogger('wg-client-thread')
log.addHandler(JournalHandler())
log.setLevel(logging.INFO)

stop_program = threading.Event()  # Event to signal program stop

def get_db_api(conc_tunnel_ip):
    try:
        db_int = db.EdgilityDatabase(user='wireguard', password='1V0eG071nP!cH', host=conc_tunnel_ip)
    except Exception as e:
        log.error("Wireguard database is currently busy, please try again later.")
        # exit(1)
    return db_int
    
# Connect to concentrator database and write timestamp
def write_to_database(server, pub_key):
    while not stop_program.is_set():
        db_int = get_db_api(str(server))
        
        current_time = int(time.time())
        values = (str(pub_key), current_time)
        try:
            db_int.insert_values_if_not_exist_otherwise_update(TIMESTAMP_POOL_NAME, TIMESTAMP_POOL_COLUMNS, values, 'date', current_time)
        except Exception as ex:
            log.error(f"Could not connect to Wireguard Concentrator database! Error: {ex}")
        
        time.sleep(60)

# Function to start the thread to write to database for the specified interface
def start_thread_for_interface(int_name, server, pub_key):
    threading.Thread(name=int_name, target=write_to_database, args=(server, pub_key,)).start()

def sigterm_handler(signum, frame):
    global stop_program
    stop_program.set()  # Set the flag to stop the program
    log.info("Received SIGTERM, stopping the program...")

def main():
    #wait 5mins before start to avoid race condition on reboot
    time.sleep(300)
    
    # Register signal handler for SIGTERM
    signal.signal(signal.SIGTERM, sigterm_handler)

    # Check if interface name is provided as command-line argument
    if len(sys.argv) < 2:
        log.error("Usage: python3 ./wg_client_thread.py interface_name server_ip our_public_key")
        sys.exit(1)
    
    interface_name = sys.argv[1]
    server_tunnel_ip = sys.argv[2]
    client_pub_key = sys.argv[3]
    
    start_thread_for_interface(interface_name, server_tunnel_ip, client_pub_key)

    # Keep the main thread alive
    while True:
        # Check if the stop signal is received
        if stop_program.is_set():
            break
        time.sleep(1)

if __name__ == "__main__":
    main()
